# -*- coding: utf-8 -*-
# Generated by Django 1.10.4 on 2017-03-08 15:06
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='SessionLookup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('user_id', models.CharField(max_length=255)),
                ('session_id', models.CharField(max_length=255)),
                ('created_at', models.DateTimeField(auto_now=True, verbose_name='Creation Date')),
                ('updated_at', models.DateTimeField(auto_now_add=True, verbose_name='Update Date')),
            ],
        ),
    ]
